@@ -4,7 +4,7 @@ local nvim_lsp = require("lspconfig")
 return {
     lsp = {
         neocmake = {
-            cmd = vim.lsp.rpc.connect('127.0.0.1', '9257'),
+            cmd = vim.lsp.rpc.connect('127.0.0.1', 9257),
             root_dir = function(fname)
                 return nvim_lsp.util.find_git_ancestor(fname)
             end,
@@ -25,6 +25,7 @@ return {
 
                 },
                 scan_cmake_in_package = false,
+                semantic_token = false,
             }
         }
     }
