@@ -1,8 +1,10 @@
 mod buildin;
 mod findpackage;
 mod includescanner;
+use crate::consts::TREESITTER_CMAKE_LANGUAGE;
 use crate::languageserver::BUFFERS_CACHE;
 use crate::scansubs::TREE_MAP;
+use crate::utils::get_node_content;
 use crate::utils::treehelper::{get_pos_type, PositionType};
 use crate::{utils, CompletionResponse};
 use buildin::{BUILDIN_COMMAND, BUILDIN_MODULE, BUILDIN_VARIABLE};
@@ -12,12 +14,16 @@ use std::sync::Arc;
 use tokio::fs;
 use tokio::sync::Mutex;
 use tower_lsp::lsp_types;
-use tower_lsp::lsp_types::{CompletionItem, CompletionItemKind, MessageType, Position};
+use tower_lsp::lsp_types::{
+    CompletionItem, CompletionItemKind, Documentation, MessageType, Position,
+};
 
 use once_cell::sync::Lazy;
 
 pub type CompleteKV = HashMap<PathBuf, Vec<CompletionItem>>;
 
+/// NOTE: collect the all completeitems in this PathBuf
+/// Include the top CMakeList.txt
 pub static COMPLETE_CACHE: Lazy<Arc<Mutex<CompleteKV>>> =
     Lazy::new(|| Arc::new(Mutex::new(HashMap::new())));
 
@@ -31,7 +37,10 @@ pub fn rst_doc_read(doc: String, filename: &str) -> Vec<CompletionItem> {
         .map(|line| CompletionItem {
             label: line.to_string(),
             kind: Some(CompletionItemKind::FUNCTION),
-            detail: Some(format!("defined command from {filename}\n{doc}")),
+            detail: Some("Command".to_string()),
+            documentation: Some(Documentation::String(format!(
+                "defined command from {filename}\n{doc}"
+            ))),
             ..Default::default()
         })
         .collect()
@@ -39,12 +48,12 @@ pub fn rst_doc_read(doc: String, filename: &str) -> Vec<CompletionItem> {
 
 pub async fn update_cache<P: AsRef<Path>>(path: P, context: &str) -> Vec<CompletionItem> {
     let mut parse = tree_sitter::Parser::new();
-    parse.set_language(&tree_sitter_cmake::language()).unwrap();
+    parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
     let thetree = parse.parse(context, None);
     let tree = thetree.unwrap();
     let Some(result_data) = getsubcomplete(
         tree.root_node(),
-        context,
+        &context.lines().collect(),
         path.as_ref(),
         PositionType::Variable,
         None,
@@ -67,27 +76,27 @@ pub async fn get_cached_completion<P: AsRef<Path>>(path: P) -> Vec<CompletionIte
     let tree_map = TREE_MAP.lock().await;
 
     while let Some(parent) = tree_map.get(&path) {
-        let complet_cache = COMPLETE_CACHE.lock().await;
-        if let Some(datas) = complet_cache.get(parent) {
-            completions.append(&mut datas.clone());
+        let complete_cache = COMPLETE_CACHE.lock().await;
+        if let Some(data) = complete_cache.get(parent) {
+            completions.append(&mut data.clone());
         } else if let Ok(context) = fs::read_to_string(parent).await {
             let mut buffer_cache = BUFFERS_CACHE.lock().await;
             buffer_cache.insert(
                 lsp_types::Url::from_file_path(parent).unwrap(),
                 context.clone(),
             );
-            drop(complet_cache);
+            drop(complete_cache);
             completions.append(&mut update_cache(parent, context.as_str()).await);
-            path = parent.clone();
+            path.clone_from(parent);
             continue;
         }
-        path = parent.clone();
+        path.clone_from(parent);
     }
 
     completions
 }
 
-/// get the complet messages
+/// get the complete messages
 pub async fn getcomplete(
     source: &str,
     location: Position,
@@ -96,7 +105,7 @@ pub async fn getcomplete(
     find_cmake_in_package: bool,
 ) -> Option<CompletionResponse> {
     let mut parse = tree_sitter::Parser::new();
-    parse.set_language(&tree_sitter_cmake::language()).unwrap();
+    parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
     let thetree = parse.parse(source, None);
     let tree = thetree.unwrap();
     let mut complete: Vec<CompletionItem> = vec![];
@@ -109,7 +118,7 @@ pub async fn getcomplete(
         PositionType::Variable | PositionType::TargetLink | PositionType::TargetInclude => {
             if let Some(mut message) = getsubcomplete(
                 tree.root_node(),
-                source,
+                &source.lines().collect(),
                 Path::new(local_path),
                 postype,
                 Some(location),
@@ -155,7 +164,7 @@ pub async fn getcomplete(
 #[allow(clippy::too_many_arguments)]
 fn getsubcomplete(
     input: tree_sitter::Node,
-    source: &str,
+    source: &Vec<&str>,
     local_path: &Path,
     postype: PositionType,
     location: Option<Position>,
@@ -169,7 +178,6 @@ fn getsubcomplete(
             return None;
         }
     }
-    let newsource: Vec<&str> = source.lines().collect();
     let mut course = input.walk();
     let mut complete: Vec<CompletionItem> = vec![];
     for child in input.children(&mut course) {
@@ -184,7 +192,7 @@ fn getsubcomplete(
                 let start_y = child.start_position().row;
                 let end_y = child.end_position().row;
                 let mut output = String::new();
-                for item in newsource.iter().take(end_y).skip(start_y + 1) {
+                for item in source.iter().take(end_y).skip(start_y + 1) {
                     output.push_str(&format!("{item}\n"));
                 }
                 complete.append(&mut rst_doc_read(
@@ -205,13 +213,17 @@ fn getsubcomplete(
                 let x = function_name.start_position().column;
                 let y = function_name.end_position().column;
                 let h = function_name.start_position().row;
-                let Some(name) = &newsource[h][x..y].split(' ').next() else {
+                let Some(name) = &source[h][x..y].split(' ').next() else {
                     continue;
                 };
                 complete.push(CompletionItem {
                     label: name.to_string(),
                     kind: Some(CompletionItemKind::FUNCTION),
-                    detail: Some(format!("defined function\nfrom: {}", local_path.display())),
+                    detail: Some("Function".to_string()),
+                    documentation: Some(Documentation::String(format!(
+                        "defined function\nfrom: {}",
+                        local_path.display()
+                    ))),
                     ..Default::default()
                 });
             }
@@ -228,14 +240,18 @@ fn getsubcomplete(
                 let x = marco_name.start_position().column;
                 let y = marco_name.end_position().column;
                 let h = marco_name.start_position().row;
-                let Some(name) = &newsource[h][x..y].split(' ').next() else {
+                let Some(name) = &source[h][x..y].split(' ').next() else {
                     continue;
                 };
 
                 complete.push(CompletionItem {
                     label: name.to_string(),
                     kind: Some(CompletionItemKind::FUNCTION),
-                    detail: Some(format!("defined function\nfrom: {}", local_path.display())),
+                    detail: Some("Function".to_string()),
+                    documentation: Some(Documentation::String(format!(
+                        "defined function\nfrom: {}",
+                        local_path.display()
+                    ))),
                     ..Default::default()
                 });
             }
@@ -274,14 +290,14 @@ fn getsubcomplete(
                 let ids = child.child(0).unwrap();
                 let x = ids.start_position().column;
                 let y = ids.end_position().column;
-                let name = newsource[h][x..y].to_lowercase();
+                let name = source[h][x..y].to_lowercase();
                 if name == "include" && child.child_count() >= 3 && should_in {
                     let ids = child.child(2).unwrap();
                     if ids.start_position().row == ids.end_position().row {
                         let h = ids.start_position().row;
                         let x = ids.start_position().column;
                         let y = ids.end_position().column;
-                        let name = &newsource[h][x..y];
+                        let name = &source[h][x..y];
                         let (is_buildin, subpath) = {
                             if name.split('.').count() != 1 {
                                 (false, local_path.parent().unwrap().join(name))
@@ -301,7 +317,7 @@ fn getsubcomplete(
                         if include_files.contains(&subpath) {
                             continue;
                         }
-                        if let Ok(true) = cmake_try_exists(&subpath) {
+                        if let Ok(true) = subpath.try_exists() {
                             if let Some(mut comps) = includescanner::scanner_include_complete(
                                 &subpath,
                                 postype,
@@ -328,14 +344,15 @@ fn getsubcomplete(
                             let startx = identifier.start_position().column;
                             let endx = identifier.end_position().column;
                             let row = identifier.start_position().row;
-                            let variable = &newsource[row][startx..endx];
+                            let variable = &source[row][startx..endx];
                             complete.push(CompletionItem {
                                 label: variable.to_string(),
                                 kind: Some(CompletionItemKind::VARIABLE),
-                                detail: Some(format!(
+                                detail: Some("Variable".to_string()),
+                                documentation: Some(Documentation::String(format!(
                                     "defined var\nfrom: {}",
                                     local_path.display()
-                                )),
+                                ))),
                                 ..Default::default()
                             });
                         }
@@ -358,7 +375,7 @@ fn getsubcomplete(
                                 let h = ids.start_position().row;
                                 let x = ids.start_position().column;
                                 let y = ids.end_position().column;
-                                let name = &newsource[h][x..y].split(' ').next();
+                                let name = &source[h][x..y].split(' ').next();
 
                                 let Some(name) = name.map(|name| name.to_string()) else {
                                     continue;
@@ -366,10 +383,11 @@ fn getsubcomplete(
                                 complete.push(CompletionItem {
                                     label: name.to_string(),
                                     kind: Some(CompletionItemKind::VALUE),
-                                    detail: Some(format!(
+                                    detail: Some("Value".to_string()),
+                                    documentation: Some(Documentation::String(format!(
                                         "defined variable\nfrom: {}",
                                         local_path.display()
-                                    )),
+                                    ))),
                                     ..Default::default()
                                 });
                             }
@@ -386,12 +404,12 @@ fn getsubcomplete(
                                 let h = package_prefix_node.start_position().row;
                                 let x = package_prefix_node.start_position().column;
                                 let y = package_prefix_node.end_position().column;
-                                let package_name = &newsource[h][x..y];
+                                let package_name = &source[h][x..y];
                                 let mut component_part = Vec::new();
                                 let mut cmakepackages = Vec::new();
                                 let components_packages = {
                                     if argument_count >= 2 {
-                                        let mut support_commponent = false;
+                                        let mut support_component = false;
                                         let mut components_packages = Vec::new();
                                         for index in 1..argument_count {
                                             let package_prefix_node =
@@ -399,16 +417,16 @@ fn getsubcomplete(
                                             let h = package_prefix_node.start_position().row;
                                             let x = package_prefix_node.start_position().column;
                                             let y = package_prefix_node.end_position().column;
-                                            let component = &newsource[h][x..y];
+                                            let component = &source[h][x..y];
                                             if component == "COMPONENTS" {
-                                                support_commponent = true;
+                                                support_component = true;
                                             } else if component != "REQUIRED" {
                                                 component_part.push(component.to_string());
                                                 components_packages
                                                     .push(format!("{package_name}::{component}"));
                                             }
                                         }
-                                        if support_commponent {
+                                        if support_component {
                                             Some(components_packages)
                                         } else {
                                             None
@@ -425,13 +443,16 @@ fn getsubcomplete(
                                 } else {
                                     cmakepackages.push(package_name.to_string());
                                 }
-                                // mordern cmake like Qt5::Core
+                                // modern cmake like Qt5::Core
                                 if let Some(components) = components_packages {
                                     for component in components {
                                         complete.push(CompletionItem {
                                             label: component,
                                             kind: Some(CompletionItemKind::VARIABLE),
-                                            detail: Some(format!("package from: {package_name}",)),
+                                            detail: Some("Variable".to_string()),
+                                            documentation: Some(Documentation::String(format!(
+                                                "package from: {package_name}",
+                                            ))),
                                             ..Default::default()
                                         });
                                     }
@@ -444,7 +465,10 @@ fn getsubcomplete(
                                     complete.push(CompletionItem {
                                         label: format!("{package_name}_LIBRARIES"),
                                         kind: Some(CompletionItemKind::VARIABLE),
-                                        detail: Some(format!("package: {package_name}",)),
+                                        detail: Some("Variable".to_string()),
+                                        documentation: Some(Documentation::String(format!(
+                                            "package: {package_name}",
+                                        ))),
                                         ..Default::default()
                                     });
                                 }
@@ -456,7 +480,10 @@ fn getsubcomplete(
                                     complete.push(CompletionItem {
                                         label: format!("{package_name}_INCLUDE_DIRS"),
                                         kind: Some(CompletionItemKind::VARIABLE),
-                                        detail: Some(format!("package: {package_name}",)),
+                                        detail: Some("Variable".to_string()),
+                                        documentation: Some(Documentation::String(format!(
+                                            "package: {package_name}",
+                                        ))),
                                         ..Default::default()
                                     });
                                 }
@@ -479,10 +506,8 @@ fn getsubcomplete(
                             #[cfg(unix)]
                             if name == "pkg_check_modules" && child.child_count() >= 3 {
                                 let ids = child.child(2).unwrap();
-                                let x = ids.start_position().column;
-                                let y = ids.end_position().column;
-                                let package_names: Vec<&str> =
-                                    newsource[h][x..y].split(' ').collect();
+                                let names = get_node_content(source, &ids);
+                                let package_names: Vec<&str> = names.split(' ').collect();
                                 let package_name = package_names[0];
 
                                 let modernpkgconfig = package_names.contains(&PKG_IMPORT_TARGET);
@@ -495,7 +520,10 @@ fn getsubcomplete(
                                     complete.push(CompletionItem {
                                         label: format!("PkgConfig::{package_name}"),
                                         kind: Some(CompletionItemKind::VARIABLE),
-                                        detail: Some(format!("package: {package_name}",)),
+                                        detail: Some("Package".to_string()),
+                                        documentation: Some(Documentation::String(format!(
+                                            "package: {package_name}",
+                                        ))),
                                         ..Default::default()
                                     });
                                 }
@@ -507,7 +535,10 @@ fn getsubcomplete(
                                     complete.push(CompletionItem {
                                         label: format!("{package_name}_LIBRARIES"),
                                         kind: Some(CompletionItemKind::VARIABLE),
-                                        detail: Some(format!("package: {package_name}",)),
+                                        detail: Some("Package".to_string()),
+                                        documentation: Some(Documentation::String(format!(
+                                            "package: {package_name}",
+                                        ))),
                                         ..Default::default()
                                     });
                                 }
@@ -518,7 +549,10 @@ fn getsubcomplete(
                                     complete.push(CompletionItem {
                                         label: format!("{package_name}_INCLUDE_DIRS"),
                                         kind: Some(CompletionItemKind::VARIABLE),
-                                        detail: Some(format!("package: {package_name}",)),
+                                        detail: Some("Package".to_string()),
+                                        documentation: Some(Documentation::String(format!(
+                                            "package: {package_name}",
+                                        ))),
                                         ..Default::default()
                                     });
                                 }
@@ -538,14 +572,6 @@ fn getsubcomplete(
     }
 }
 
-fn cmake_try_exists(input: &PathBuf) -> std::io::Result<bool> {
-    match std::fs::metadata(input) {
-        Ok(_) => Ok(true),
-        Err(error) if error.kind() == std::io::ErrorKind::NotFound => Ok(false),
-        Err(error) => Err(error),
-    }
-}
-
 fn get_cmake_package_complete(
     package_name: &str,
     postype: PositionType,
