@@ -5,16 +5,20 @@ use self::config::Config;
 use super::Backend;
 use crate::ast;
 use crate::complete;
+use crate::consts::TREESITTER_CMAKE_LANGUAGE;
 use crate::filewatcher;
 use crate::formatting::getformat;
 use crate::gammar::checkerror;
+use crate::gammar::LintConfigInfo;
 use crate::jump;
 use crate::scansubs;
+use crate::semantic_token;
+use crate::semantic_token::LEGEND_TYPE;
 use crate::utils::treehelper;
-use serde_json::Value;
 use std::collections::HashMap;
 use std::path::Path;
 use std::sync::Arc;
+use std::sync::RwLock;
 use tokio::sync::Mutex;
 use tower_lsp::jsonrpc::Result;
 use tower_lsp::lsp_types;
@@ -27,15 +31,38 @@ use once_cell::sync::Lazy;
 pub static BUFFERS_CACHE: Lazy<Arc<Mutex<HashMap<lsp_types::Url, String>>>> =
     Lazy::new(|| Arc::new(Mutex::new(HashMap::new())));
 
+static CLIENT_CAPABILITIES: RwLock<Option<TextDocumentClientCapabilities>> = RwLock::new(None);
+
+fn set_client_text_document(text_document: Option<TextDocumentClientCapabilities>) {
+    let mut data = CLIENT_CAPABILITIES.write().unwrap();
+    *data = text_document;
+}
+
+pub fn get_client_capabilities() -> Option<TextDocumentClientCapabilities> {
+    let data = CLIENT_CAPABILITIES.read().unwrap();
+    data.clone()
+}
+
+pub fn client_support_snippet() -> bool {
+    match get_client_capabilities() {
+        Some(c) => c
+            .completion
+            .and_then(|item| item.completion_item)
+            .and_then(|item| item.snippet_support)
+            .unwrap_or(false),
+        _ => false,
+    }
+}
+
 impl Backend {
-    async fn publish_diagnostics(&self, uri: Url, context: String) {
+    async fn publish_diagnostics(&self, uri: Url, context: String, lint_info: LintConfigInfo) {
         let mut parse = Parser::new();
-        parse.set_language(&tree_sitter_cmake::language()).unwrap();
+        parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
         let thetree = parse.parse(&context, None);
         let Some(tree) = thetree else {
             return;
         };
-        let gammererror = checkerror(Path::new(uri.path()), &context, tree.root_node());
+        let gammererror = checkerror(Path::new(uri.path()), &context, tree.root_node(), lint_info);
         if let Some(diagnoses) = gammererror {
             let mut pusheddiagnoses = vec![];
             for (start, end, message, severity) in diagnoses.inner {
@@ -68,34 +95,44 @@ impl Backend {
     async fn update_diagnostics(&self) {
         let storemap = BUFFERS_CACHE.lock().await;
         for (uri, context) in storemap.iter() {
-            self.publish_diagnostics(uri.clone(), context.to_string())
-                .await;
+            self.publish_diagnostics(
+                uri.clone(),
+                context.to_string(),
+                LintConfigInfo {
+                    use_lint: self.init_info.lock().await.enable_lint,
+                    use_extra_cmake_lint: true,
+                },
+            )
+            .await;
         }
     }
 }
 
 #[tower_lsp::async_trait]
 impl LanguageServer for Backend {
-    async fn initialize(&self, inital: InitializeParams) -> Result<InitializeResult> {
-        let inital_config: Config = inital
+    async fn initialize(&self, initial: InitializeParams) -> Result<InitializeResult> {
+        let initial_config: Config = initial
             .initialization_options
             .and_then(|value| serde_json::from_value(value).unwrap_or(None))
             .unwrap_or_default();
 
-        let do_format = inital_config.is_format_enabled();
+        let do_format = initial_config.is_format_enabled();
 
-        let find_cmake_in_package = inital_config.is_scan_cmake_in_package();
+        let find_cmake_in_package = initial_config.is_scan_cmake_in_package();
+
+        let lint_enable = initial_config.is_lint_enabled();
 
         let mut init_info = self.init_info.lock().await;
         init_info.scan_cmake_in_package = find_cmake_in_package;
+        init_info.enable_lint = lint_enable;
 
-        if let Some(workspace) = inital.capabilities.workspace {
+        if let Some(workspace) = initial.capabilities.workspace {
             if let Some(watch_file) = workspace.did_change_watched_files {
                 if let (Some(true), Some(true)) = (
                     watch_file.dynamic_registration,
                     watch_file.relative_pattern_support,
                 ) {
-                    if let Some(ref uri) = inital.root_uri {
+                    if let Some(ref uri) = initial.root_uri {
                         let path = std::path::Path::new(uri.path())
                             .join("build")
                             .join("CMakeCache.txt");
@@ -107,17 +144,29 @@ impl LanguageServer for Backend {
             }
         }
 
-        if let Some(ref uri) = inital.root_uri {
+        if let Some(ref uri) = initial.root_uri {
             scansubs::scan_all(uri.path()).await;
             let mut root_path = self.root_path.lock().await;
             root_path.replace(uri.path().into());
         }
 
+        set_client_text_document(initial.capabilities.text_document);
+
+        let version: String = env!("CARGO_PKG_VERSION").to_string();
         Ok(InitializeResult {
-            server_info: None,
+            server_info: Some(ServerInfo {
+                name: "neocmakelsp".to_string(),
+                version: Some(version),
+            }),
             capabilities: ServerCapabilities {
-                text_document_sync: Some(TextDocumentSyncCapability::Kind(
-                    TextDocumentSyncKind::FULL,
+                text_document_sync: Some(TextDocumentSyncCapability::Options(
+                    TextDocumentSyncOptions {
+                        open_close: Some(true),
+                        change: Some(TextDocumentSyncKind::FULL),
+                        will_save: Some(false),
+                        will_save_wait_until: Some(false),
+                        save: Some(TextDocumentSyncSaveOptions::Supported(true)),
+                    },
                 )),
                 completion_provider: Some(CompletionOptions {
                     resolve_provider: Some(false),
@@ -126,10 +175,6 @@ impl LanguageServer for Backend {
                     all_commit_characters: None,
                     completion_item: None,
                 }),
-                execute_command_provider: Some(ExecuteCommandOptions {
-                    commands: vec!["dummy.do_something".to_string()],
-                    work_done_progress_options: Default::default(),
-                }),
                 document_symbol_provider: Some(OneOf::Left(true)),
                 definition_provider: Some(OneOf::Left(true)),
                 document_formatting_provider: if do_format {
@@ -145,6 +190,35 @@ impl LanguageServer for Backend {
                     }),
                     file_operations: None,
                 }),
+                semantic_tokens_provider: if initial_config.enable_semantic_token() {
+                    Some(
+                        SemanticTokensServerCapabilities::SemanticTokensRegistrationOptions(
+                            SemanticTokensRegistrationOptions {
+                                text_document_registration_options: {
+                                    TextDocumentRegistrationOptions {
+                                        document_selector: Some(vec![DocumentFilter {
+                                            language: Some("cmake".to_string()),
+                                            scheme: Some("file".to_string()),
+                                            pattern: None,
+                                        }]),
+                                    }
+                                },
+                                semantic_tokens_options: SemanticTokensOptions {
+                                    work_done_progress_options: WorkDoneProgressOptions::default(),
+                                    legend: SemanticTokensLegend {
+                                        token_types: LEGEND_TYPE.into(),
+                                        token_modifiers: vec![],
+                                    },
+                                    range: None,
+                                    full: Some(SemanticTokensFullOptions::Bool(true)),
+                                },
+                                static_registration_options: StaticRegistrationOptions::default(),
+                            },
+                        ),
+                    )
+                } else {
+                    None
+                },
                 references_provider: Some(OneOf::Left(true)),
                 ..ServerCapabilities::default()
             },
@@ -206,7 +280,9 @@ impl LanguageServer for Backend {
                 scansubs::scan_all(path).await;
                 continue;
             }
-            tracing::info!("CMakeCache changed");
+            self.client
+                .log_message(MessageType::INFO, "CMakeCache changed")
+                .await;
             if let FileChangeType::DELETED = change.typ {
                 filewatcher::clear_error_packages();
             } else {
@@ -220,28 +296,22 @@ impl LanguageServer for Backend {
             .await;
     }
 
-    async fn execute_command(&self, _: ExecuteCommandParams) -> Result<Option<Value>> {
-        self.client
-            .log_message(MessageType::INFO, "command executed!")
-            .await;
-
-        match self.client.apply_edit(WorkspaceEdit::default()).await {
-            Ok(res) if res.applied => self.client.log_message(MessageType::INFO, "applied").await,
-            Ok(_) => self.client.log_message(MessageType::INFO, "rejected").await,
-            Err(err) => self.client.log_message(MessageType::ERROR, err).await,
-        }
-
-        Ok(None)
-    }
-
     async fn did_open(&self, input: DidOpenTextDocumentParams) {
         let mut parse = Parser::new();
-        parse.set_language(&tree_sitter_cmake::language()).unwrap();
+        parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
         let uri = input.text_document.uri.clone();
         let context = input.text_document.text.clone();
         let mut storemap = BUFFERS_CACHE.lock().await;
         storemap.entry(uri.clone()).or_insert(context.clone());
-        self.publish_diagnostics(uri, context).await;
+        self.publish_diagnostics(
+            uri,
+            context,
+            LintConfigInfo {
+                use_lint: self.init_info.lock().await.enable_lint,
+                use_extra_cmake_lint: true,
+            },
+        )
+        .await;
         self.client
             .log_message(MessageType::INFO, "file opened!")
             .await;
@@ -254,7 +324,15 @@ impl LanguageServer for Backend {
         let mut storemap = BUFFERS_CACHE.lock().await;
         storemap.insert(uri.clone(), context.clone());
         if context.lines().count() < 500 {
-            self.publish_diagnostics(uri, context).await;
+            self.publish_diagnostics(
+                uri,
+                context,
+                LintConfigInfo {
+                    use_lint: self.init_info.lock().await.enable_lint,
+                    use_extra_cmake_lint: false,
+                },
+            )
+            .await;
         }
         self.client
             .log_message(MessageType::INFO, &format!("{input:?}"))
@@ -274,26 +352,21 @@ impl LanguageServer for Backend {
             if has_root {
                 complete::update_cache(uri.path(), context).await;
             }
-            self.publish_diagnostics(uri, context.to_string()).await;
+            self.publish_diagnostics(
+                uri,
+                context.to_string(),
+                LintConfigInfo {
+                    use_lint: self.init_info.lock().await.enable_lint,
+                    use_extra_cmake_lint: true,
+                },
+            )
+            .await;
         }
         self.client
             .log_message(MessageType::INFO, "file saved!")
             .await;
     }
 
-    async fn signature_help(&self, _: SignatureHelpParams) -> Result<Option<SignatureHelp>> {
-        Ok(Some(SignatureHelp {
-            signatures: vec![SignatureInformation {
-                label: "Test".to_string(),
-                documentation: None,
-                parameters: None,
-                active_parameter: None,
-            }],
-            active_signature: None,
-            active_parameter: None,
-        }))
-    }
-
     async fn hover(&self, params: HoverParams) -> Result<Option<Hover>> {
         let position = params.text_document_position_params.position;
         let uri = params.text_document_position_params.text_document.uri;
@@ -303,7 +376,7 @@ impl LanguageServer for Backend {
         match storemap.get(&uri) {
             Some(context) => {
                 let mut parse = Parser::new();
-                parse.set_language(&tree_sitter_cmake::language()).unwrap();
+                parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
                 let thetree = parse.parse(context.clone(), None);
                 let tree = thetree.unwrap();
                 let output = treehelper::get_cmake_doc(position, tree.root_node(), context);
@@ -328,7 +401,7 @@ impl LanguageServer for Backend {
         self.client
             .log_message(
                 MessageType::INFO,
-                format!("formating, space is {}", input.options.insert_spaces),
+                format!("formatting, space is {}", input.options.insert_spaces),
             )
             .await;
         let uri = input.text_document.uri;
@@ -386,9 +459,16 @@ impl LanguageServer for Backend {
         match storemap.get(&uri) {
             Some(context) => {
                 let mut parse = Parser::new();
-                parse.set_language(&tree_sitter_cmake::language()).unwrap();
+                parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
                 //notify_send(context, Type::Error);
-                Ok(jump::godef(location, context, uri.path().to_string(), &self.client).await)
+                Ok(jump::godef(
+                    location,
+                    context,
+                    uri.path().to_string(),
+                    &self.client,
+                    false,
+                )
+                .await)
             }
             None => Ok(None),
         }
@@ -403,14 +483,22 @@ impl LanguageServer for Backend {
         match storemap.get(&uri) {
             Some(context) => {
                 let mut parse = Parser::new();
-                parse.set_language(&tree_sitter_cmake::language()).unwrap();
+                parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
                 let thetree = parse.parse(context.clone(), None);
                 let tree = thetree.unwrap();
                 let origin_selection_range =
                     treehelper::get_position_range(location, tree.root_node());
 
                 //notify_send(context, Type::Error);
-                match jump::godef(location, context, uri.path().to_string(), &self.client).await {
+                match jump::godef(
+                    location,
+                    context,
+                    uri.path().to_string(),
+                    &self.client,
+                    true,
+                )
+                .await
+                {
                     Some(range) => Ok(Some(GotoDefinitionResponse::Link({
                         range
                             .iter()
@@ -446,4 +534,18 @@ impl LanguageServer for Backend {
             None => Ok(None),
         }
     }
+    async fn semantic_tokens_full(
+        &self,
+        params: SemanticTokensParams,
+    ) -> Result<Option<SemanticTokensResult>> {
+        let uri = params.text_document.uri.clone();
+        self.client
+            .log_message(MessageType::LOG, "semantic_token_full")
+            .await;
+        let storemap = BUFFERS_CACHE.lock().await;
+        match storemap.get(&uri) {
+            Some(context) => Ok(semantic_token::semantic_token(&self.client, context).await),
+            None => Ok(None),
+        }
+    }
 }
